#   This file is part of Gnupylot.
#
#    Gnupylot is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Gnupylot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Gnupylot; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
Created on 20/gen/2010

@author: Lorenzo Rovigatti
'''
from PyQt4 import QtCore

try:
    from lib import common, paths
except ImportError:
    from src.lib import common, paths

# I got the idea behind this Config stuff from emesene.
# FIXME: it appears that the DEFAULT_CONFIG is overwriting the config file
DEFAULT_CONFIG = {
                'actionsInToolbar' : 'new,save,load,close,separator,replot,separator,undo,redo',
                'replotAfterChanges' : 1,
                'gnuplotWidgetErrorColor' : '#FF0000',
                'gnuplotWidgetCommandColor' : '#000000',
                'gnuplotWidgetOutputColor' : '#0000FF',
                'gnuplotBin' : 'gnuplot',
                'printCommand' : 'lpr'
                }

class Config(QtCore.QObject):
    """Application's config manager.
    It load the configuration from the config file and save it after each change.
    """
    
    def __init__(self):
        configDir = QtCore.QDir(paths.getConfigDir())
        if not configDir.exists(): configDir.mkdir(paths.getConfigDir())
        
        self.configFile = QtCore.QFile(paths.getConfigDir() + "config")
        if not self.configFile.exists():
            self.config = ConfigDict(self, self.writeConfig, DEFAULT_CONFIG)
            self.writeConfig()
            
        self.readConfig()
        
    def writeConfig(self):
        '''Write the config to the file, overwriting current config file.'''
        try:
            self.configFile.open(self.configFile.WriteOnly)

            for k, v in self.config:
                if type(v) == bool:
                    self.configFile.write(k + '=' + str(int(v)) + '\n')
                else:
                    self.configFile.write(k + '=' + str(v) + '\n')

            self.configFile.close()
        except Exception, e:
            self.configFile.close()
            common.debug('exception writing config:' + e)
    
    def readConfig(self):
        '''Read the config file and create a dictionary that stores the key=value couples
        of the config file.'''
        configDict = DEFAULT_CONFIG.copy()
        try:
            self.configFile.open(self.configFile.ReadOnly)
            content = QtCore.QString(self.configFile.readAll())

            for line in content.split("\n"):
                if line != '':
                    try:
                        delim = line.find('=')
                        key = line[:delim]
                        value = line[delim + 1:]
                        if key in DEFAULT_CONFIG:
                            configDict[key] = value
                        else:
                            common.debug(key + ' is not a valid config key, ignored')
                    except Exception:
                        common.debug(key + ' config value is incorrect')

            self.configFile.close()
        except:
            if self.configFile.isOpen(): self.configFile.close()

        self.config = ConfigDict(self, self.writeConfig, DEFAULT_CONFIG, configDict)

    
class ConfigDict(dict):
    '''A dictionary that handles type conversion, fallbacks to default, config
    file writing, etc'''

    def __init__(self, config, writeFunc, defaults, initFrom=None):
        self.config = config
        self.writeFunc = writeFunc
        self.defaults = defaults

        if initFrom == None: initFrom = defaults.copy()

        self.store = {}
        for key, value in initFrom.iteritems():
            if type(value) == bool:
                self.store[key] = str(int(value))
            else:
                self.store[key] = str(value)

    def getDefault(self, name):
        if name in self.defaults:
            return self.defaults[name]
        else:
            return ''

    def __getitem__(self, name):
        """Returns a item from self.store. If it is not available, returns that
        item from defaults.
        """

        if name in self.store:
            return self.store[name]
        else:
            common.debug('Config value was not found: ' + str(name))
            return self.getDefault(name)

    def __setitem__(self, name, value):
        '''Sets a config key "name" to the string "value" and writes config.'''

        if type(value) == bool:
            value = str(int(value))
        else:
            value = str(value)

        oldValue = None
        if name in self.store:
            oldValue = self.store[name]

        common.debug("Changed " + name + " from '" + oldValue + "' to '" + value + "'")
        self.store[name] = value
        self.writeFunc()

    def __iter__(self):
        '''For key, value in ConfigDict.'''
        return self.store.iteritems()
