#   This file is part of Gnupylot.
#
#    Gnupylot is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Gnupylot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Gnupylot; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
Created on 25/gen/2010

@author: Lorenzo Rovigatti
'''

from PyQt4 import QtCore, QtGui
from UI import SettingsDock

try:
    from lib import UndoCommands
except ImportError:
    from src.lib import UndoCommands

class SettingsDockWidget(QtGui.QDockWidget, SettingsDock.Ui_SettingsDock):
    """A dockbar widget used to set the current plot's settings."""
    def __init__(self, controller, parent):
        QtGui.QDockWidget.__init__(self, parent)
        self.setupUi(self)
        self.controller = controller
        """Reference to the current L{Controller}.
        @type: L{Controller}"""
        
        # TODO: add a specific validator for these fields
#        validator = QtGui.QDoubleValidator(self)
#        self.xfrom.setValidator(validator)
#        self.xto.setValidator(validator)
#        self.yfrom.setValidator(validator)
#        self.yto.setValidator(validator)
        
        self.connect(self.updatePlotButton, QtCore.SIGNAL('clicked()'), self.updatePlot)
        self.connect(self, QtCore.SIGNAL('plotToUpdate'), parent.updatePlot)
        
    def getTitle(self):
        return str(self.plotTitle.text())
        
    def getXLabel(self):
        return str(self.xlabel.text())
    
    def getYLabel(self):
        return str(self.ylabel.text())
    
    def getXFrom(self):
        return str(self.xfrom.text())
    
    def getXTo(self):
        return str(self.xto.text())
    
    def getYFrom(self):
        return str(self.yfrom.text())
    
    def getYTo(self):
        return str(self.yto.text())
        
    def updatePlot(self):
        '''
        Update settings in the current plot.
        '''
        settings = {}
        settings['title'] = self.getTitle()
        settings['xlabel'] = self.getXLabel()
        settings['ylabel'] = self.getYLabel()
        xfrom = self.getXFrom()
        xto = self.getXTo()
        yfrom = self.getYFrom()
        yto = self.getYTo()
#        if len(xfrom) == 0: xfrom = '*'
#        if len(xto) == 0: xto = '*'
#        if len(yfrom) == 0: yfrom = '*'
#        if len(yto) == 0: yto = '*'
        settings['xrange'] = (xfrom, xto)
        settings['yrange'] = (yfrom, yto)
        settings['xlogscale'] = self.xlog.isChecked()
        settings['ylogscale'] = self.ylog.isChecked()
        
        plot = self.controller.getCurrentPlot()
        comm = UndoCommands.ChangePlotSettingsCommand(plot, settings)
        plot.pushCommand(comm)

    def loadSettings(self):
        """Load the given settings to the widgets of the dockbar."""
        settings = self.controller.getCurrentPlot().settings
        
        self.plotTitle.setText(settings['title'])
        self.xlabel.setText(settings['xlabel'])
        self.ylabel.setText(settings['ylabel'])

        self.xfrom.setText(settings['xrange'][0])
        self.xto.setText(settings['xrange'][1])
        self.yfrom.setText(settings['yrange'][0])
        self.yto.setText(settings['yrange'][1])
        
        self.xlog.setChecked(settings['xlogscale'])
        self.ylog.setChecked(settings['ylogscale'])
        