#   This file is part of Gnupylot.
#
#    Gnupylot is free software; you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation; either version 2 of the License, or
#    (at your option) any later version.
#
#    Gnupylot is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with Gnupylot; if not, write to the Free Software
#    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
'''
Created on 04/mag/2010

@author: lorenzo
'''
from PyQt4 import QtCore
from numpy import *

try:
    from Gnuplot import utils
except ImportError:
    from src.Gnuplot import utils

class DataFile(QtCore.QFile):
    '''
    Incapsulates a file containing data
    '''


    def __init__(self, parent):
        '''
        Constructor.
        '''
        QtCore.QFile.__init__(self, parent)

        self.availableDatasets = []
        """Data read from file.
        It's divided in sets, so it's a list of arrays that contain the actual data.
        @type: list of numpy arrays"""
        self.filteredDatasets = []
        """Data filtered using user choosen columns or expressions.
        @type: list of numpy arrays"""
        self.nCols = 0
        """Number of columns in file
        @type: int
        """

    def parseFile(self, filename):
        """Read the given filename, parsing it to check if it's well formed and to count how many columns has."""
        self.setFileName(filename)
        if not self.exists(): return False

        self.availableDatasets = []

        # let's count how many columns are in the file. To do this we loop over
        # the file until we find a non commented line and we split it 
        self.open(self.ReadOnly)
        first = True
        halfFirst = False
        index = -1
        counter = 0
        while self.atEnd() == False:
            # python strings are way better to handle in this case
            line = str(self.readLine()).strip()
            if not line.startswith('#'):
                # if there are two or more consecutive spaces between lines
                # then set first = True, this will add a dataset.
                if len(line) == 0:
                    if halfFirst == True:
                        first = True
                    else: halfFirst = True
                else:
                    halfFirst = False
                    cols = [float(x) for x in line.split()]
                    # the number of columns is given by the first line
                    if first == True:
                        self.availableDatasets.append([])
                        index += 1
                        nCols = len(cols)
                        first = False
                        counter = 0

                    # every item of self.availableDatasets[index] must be of nCols length:
                    # if it's shorter pad it with 0s, if it's longer cut the last elements
                    actNCols = len(cols)
                    if actNCols > nCols: cols = cols[:nCols]
                    if actNCols < nCols: cols += [0, ] * (nCols - actNCols)

                    cols.insert(0, float(counter))
                    self.availableDatasets[index].append(cols)
                    counter += 1

        self.close()

        # convert datasets in numpy arrays
        for i in range(0, len(self.availableDatasets)):
            self.availableDatasets[i] = utils.float_array(self.availableDatasets[i])

        res = [str(x) for x in range(1, nCols + 1)]
        self.nCols = nCols

        return res

    def filterDatasets(self, cols=None):
        """Populate L{self.filteredDatasets} with the columns (or with the resulting columns if expressions are used) given as 
        second arguments.
        @param cols: (optional) columns (or expressions dealing with columns). If it's not passed then the first two columns (or
        index + first column) will be used.
        @type cols: tuple"""

        # TODO: we have to check if self.availableDatasets is empty. If it's empty we have to raise an exception
        self.filteredDatasets = []

        if cols == None:
            if self.nCols == 1: cols = ["$ind", "$1"]
            else: cols = ["$1", "$2"]

        for j in range(0, len(cols)):
            cols[j] = cols[j].replace("$ind", "(data[:,0])")
            # FIXME: dirty method. We want to substitute every $i the i-th column. This will work
            # for files with less than 50 columns. It should be enough but it's dirty.
            for i in range(1, 50):
                cols[j] = cols[j].replace("$" + str(i), "(data[:," + str(i) + "])")

        for data in self.availableDatasets:
            stackCmd = "column_stack((" + ",".join(cols) + "))"
            self.filteredDatasets.append(eval(stackCmd))
